package scanner;

import stream.SafeInputStream;
import util.Characters;

import java.io.InputStream;

/**
 * Removes Lisp comments from an input stream.
 */
public class LispCommentRemovingInputStream implements LispInputStream {

    private SafeInputStream underlyingInputStream;
    private boolean isInQuotedString;
    private boolean rereadLastCharacter;
    private int previousCharacter;
    private int currentCharacter;

    public LispCommentRemovingInputStream(InputStream underlyingInputStream) {
        this.underlyingInputStream = new SafeInputStream(underlyingInputStream);
        this.isInQuotedString = false;
        this.rereadLastCharacter = false;
        this.previousCharacter = 0;
        this.currentCharacter = 0;
    }

    @Override
    public int read() {
        if (!rereadLastCharacter)
            return readFromUnderlyingInputStream();

        rereadLastCharacter = false;

        return currentCharacter;
    }

    private int readFromUnderlyingInputStream() {
        readNextCharacter();

        if (haveEnteredComment())
            consumeAllBytesInComment();

        return currentCharacter;
    }

    private void readNextCharacter() {
        previousCharacter = currentCharacter;
        currentCharacter = underlyingInputStream.read();

        if (haveEncounteredStringBoundary())
            isInQuotedString = !isInQuotedString;
    }

    private boolean haveEncounteredStringBoundary() {
        return (previousCharacter != Characters.BACKSLASH) && (currentCharacter == Characters.DOUBLE_QUOTE);
    }

    private boolean haveEnteredComment() {
        return (currentCharacter == Characters.SEMICOLON) && (!isInQuotedString);
    }

    private void consumeAllBytesInComment() {
        while (stillInComment())
            currentCharacter = underlyingInputStream.read();
    }

    private boolean stillInComment() {
        return (currentCharacter != Characters.NEWLINE) && (currentCharacter != Characters.EOF);
    }

    @Override
    public void unreadLastCharacter() {
        if (rereadLastCharacter)
            throw new MaximumUnreadsExceededException();

        this.rereadLastCharacter = true;
    }
}
